import java.awt.*;
import java.applet.*;


//Class RLCrf
public class RLCrf extends Applet {
    
	//Fields
	RLCrfControls controls;
	
	//Constructor
	public void init() {
		setLayout(new BorderLayout());
		RLCrfCanvas c = new RLCrfCanvas();
		add("Center", c);
		add("East", controls = new RLCrfControls(c));
    }

    //Methods
	public void start() {
		controls.enable();
    }

    public void stop() {
		controls.disable();
    }

    public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY) {
			System.exit(0);
		}
		return false;
    }

    public static void main(String args[]) {
		Frame f = new Frame("Simulador da Resposta em Frequncia de Circuitos");
		RLCrf	RLCrfSimulator = new RLCrf();

		RLCrfSimulator.init();
		RLCrfSimulator.start();

		f.add("Center", RLCrfSimulator);
		f.resize(590, 610);
		f.show();
    }
}
    

//Class RLCrfCanvas
class RLCrfCanvas extends Canvas {
    
	//Fields
	final static int GRAPHICS_WIDTH = 360;
	final static int GRAPHICS_HEIGHT = 280;
	final static int DRAWING_WIDTH = 320;
	final static int DRAWING_HEIGHT = 240;
	final static int SEPARATOR_SIZE = 20;
	final static int GRID_SIZE = 20;
	float[] amp_pts = new float[DRAWING_WIDTH];
	float[] pha_pts = new float[DRAWING_WIDTH];
	boolean draw_flag = false;//true when graph is valid

	//Constructor
	public RLCrfCanvas() {
		setBackground(Color.white);
	}

	//Methods
	public void paint(Graphics g) {
		Rectangle r = new Rectangle(GRAPHICS_WIDTH, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE);

		int hlines = GRAPHICS_HEIGHT / GRID_SIZE;
		int vlines = GRAPHICS_WIDTH / GRID_SIZE;
		//Grid 1
		g.setColor(new Color(211, 211, 211));
		for (int i = 1; i < hlines; i++)
		    g.drawLine(0, i * GRID_SIZE, GRAPHICS_WIDTH, i * GRID_SIZE);
		for (int i = 1; i < vlines; i++)
			g.drawLine(i * GRID_SIZE, 0, i * GRID_SIZE, GRAPHICS_HEIGHT);
		
		//Grid 2
		g.setColor(new Color(211, 211, 211));
		for (int i = 1; i < hlines; i++)
		    g.drawLine(0, i * GRID_SIZE + GRAPHICS_HEIGHT + SEPARATOR_SIZE, GRAPHICS_WIDTH, i * GRID_SIZE + GRAPHICS_HEIGHT + SEPARATOR_SIZE);
		for (int i = 1; i < vlines; i++)
			g.drawLine(i * GRID_SIZE, GRAPHICS_HEIGHT + SEPARATOR_SIZE, i * GRID_SIZE, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE);
		
		//Border 1
		g.setColor(new Color(0, 0, 128));
		g.drawLine(0, 0, GRAPHICS_WIDTH, 0);
		g.drawLine(0, GRAPHICS_HEIGHT, GRAPHICS_WIDTH, GRAPHICS_HEIGHT);
		g.drawLine(0, 0, 0, GRAPHICS_HEIGHT);
		g.drawLine(GRAPHICS_WIDTH, 0, GRAPHICS_WIDTH, GRAPHICS_HEIGHT);

		//Border 2
		g.setColor(new Color(0, 0, 128));
		g.drawLine(0, GRAPHICS_HEIGHT + SEPARATOR_SIZE, GRAPHICS_WIDTH, GRAPHICS_HEIGHT + SEPARATOR_SIZE);
		g.drawLine(0, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE, GRAPHICS_WIDTH, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE);
		g.drawLine(0, GRAPHICS_HEIGHT + SEPARATOR_SIZE, 0, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE);
		g.drawLine(GRAPHICS_WIDTH, GRAPHICS_HEIGHT + SEPARATOR_SIZE, GRAPHICS_WIDTH, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE);
		
		int[] xpoints = new int[3];
		int[] ypoints = new int[3];
		//Axis 1
		g.setColor(Color.black);
		//Up Arrow
		xpoints[0] = 20;
		ypoints[0] = 0;
		xpoints[1] = 16;
		ypoints[1] = 8;
		xpoints[2] = 24;
		ypoints[2] = 8;
		g.fillPolygon(xpoints, ypoints, 3);
		//Right Arrow
		xpoints[0] = GRAPHICS_WIDTH;
		ypoints[0] = GRAPHICS_HEIGHT / 2;
		xpoints[1] = GRAPHICS_WIDTH - 8;
		ypoints[1] = GRAPHICS_HEIGHT / 2 - 4;
		xpoints[2] = GRAPHICS_WIDTH - 8;
		ypoints[2] = GRAPHICS_HEIGHT / 2 + 4;
		g.fillPolygon(xpoints, ypoints, 3);
		//X-axis
		g.drawLine(0, GRAPHICS_HEIGHT / 2, GRAPHICS_WIDTH, GRAPHICS_HEIGHT / 2);
		//Y-axis
		g.drawLine(20, 0, 20, GRAPHICS_HEIGHT);
		
		//Axis 2
		g.setColor(Color.black);
		//Up Arrow
		xpoints[0] = 20;
		ypoints[0] = GRAPHICS_HEIGHT + SEPARATOR_SIZE;
		xpoints[1] = 16;
		ypoints[1] = GRAPHICS_HEIGHT + SEPARATOR_SIZE + 8;
		xpoints[2] = 24;
		ypoints[2] = GRAPHICS_HEIGHT + SEPARATOR_SIZE + 8;
		g.fillPolygon(xpoints, ypoints, 3);
		//Right Arrow
		xpoints[0] = GRAPHICS_WIDTH;
		ypoints[0] = GRAPHICS_HEIGHT + SEPARATOR_SIZE + GRAPHICS_HEIGHT / 2;
		xpoints[1] = GRAPHICS_WIDTH - 8;
		ypoints[1] = GRAPHICS_HEIGHT + SEPARATOR_SIZE + GRAPHICS_HEIGHT / 2 - 4;
		xpoints[2] = GRAPHICS_WIDTH - 8;
		ypoints[2] = GRAPHICS_HEIGHT + SEPARATOR_SIZE + GRAPHICS_HEIGHT / 2 + 4;
		g.fillPolygon(xpoints, ypoints, 3);
		//X-axis
		g.drawLine(0, GRAPHICS_HEIGHT + SEPARATOR_SIZE + GRAPHICS_HEIGHT / 2, GRAPHICS_WIDTH, GRAPHICS_HEIGHT + SEPARATOR_SIZE + GRAPHICS_HEIGHT / 2);
		//Y-axis
		g.drawLine(20, GRAPHICS_HEIGHT + SEPARATOR_SIZE, 20, 2 * GRAPHICS_HEIGHT + SEPARATOR_SIZE);

		Font graphics_font = new Font("Arial", Font.PLAIN, 12);
		g.setFont(graphics_font);
		//Text 1
		g.setColor(Color.red);
		g.drawString("H (f)", 25, 15); 
		g.setColor(Color.black);
		g.drawString("f", GRAPHICS_WIDTH - 10, GRAPHICS_HEIGHT / 2 + 15); 

		//Text 2
		g.setColor(Color.red);
		g.drawString("\u03c6 (f)", 25, 315); 
		g.setColor(Color.black);
		g.drawString("f", GRAPHICS_WIDTH - 10, GRAPHICS_HEIGHT + SEPARATOR_SIZE + GRAPHICS_HEIGHT / 2 + 15); 
		
		//Drawing
		if (draw_flag) { 
			g.setColor(Color.red);
			for (int i = 0; i < DRAWING_WIDTH; i++) {
				g.drawLine(i + 20, (int) amp_pts[i], i + 20, (int) amp_pts[i]);
				g.drawLine(i + 20, (int) pha_pts[i], i + 20, (int) pha_pts[i]);
			}
		}
	}
	
	void setDrawFlag(boolean draw_flag) {
		this.draw_flag = draw_flag;
	}

	void realToScreen(Label y1, Label y2) {
	   float max_amp = 0, amp_gain = 0, amp_offset, 
			 max_pha = 0, pha_gain = 0, pha_offset;

	   //Computing maximum amp and ph for graphics auto-arrange in Y
	   for (int i = 0; i < DRAWING_WIDTH; i++) {
		   if (Math.abs(amp_pts[i]) > max_amp)
				max_amp = Math.abs(amp_pts[i]);
	   	   if (Math.abs(pha_pts[i]) > max_pha)
				max_pha = Math.abs(pha_pts[i]);
	   }	   

	   	//Values for conversion	from real output to screen coordinates
		if (max_amp > 1e-3) {
			amp_gain = DRAWING_HEIGHT / (2 * max_amp);
			//Print values for Y1 divisions
			y1.setText(Float.toString(1 / amp_gain * GRID_SIZE));
		}
		amp_offset = DRAWING_HEIGHT / 2 + 20;
		
		if (max_pha > 1e-3) {
			pha_gain = DRAWING_HEIGHT / (2 * max_pha);
			//Print values for Y2 divisions
			y2.setText(Float.toString(1 / pha_gain * GRID_SIZE));
		}
		pha_offset = DRAWING_HEIGHT / 2 + GRAPHICS_HEIGHT + SEPARATOR_SIZE + 20;

		//Conversion from real values to screen coordinates
		for (int i = 0; i < DRAWING_WIDTH; i++) {
			amp_pts[i] = amp_offset - amp_gain * amp_pts[i];			
			pha_pts[i] = pha_offset - pha_gain * pha_pts[i];			
		}
		
		//Draw Graphics
		setDrawFlag(true);
		repaint();
   }
}


//Class RLCrfControls
class RLCrfControls extends Panel {
    
	//Fields
	RLCrfCanvas canvas;
	TextField tf_k, tf_zo, tf_po, tf_fmin, tf_fmax;
	TextField[] tf_zr = new TextField[4];//[f] 
	TextField[] tf_pr = new TextField[4];
	TextField[][] tf_zcc = new TextField[2][2];//[f][Q]
	TextField[][] tf_pcc = new TextField[2][2];
	Choice c_szr12 = new Choice();  
	Choice c_szr34 = new Choice();  
	Choice c_szcc1 = new Choice();
	Choice c_szcc2 = new Choice();
	Label l_error1, l_error2, l_divx, l_divy1, l_divy2;
	float ln10 = (float) Math.log(10);
	float[] freq = new float[canvas.DRAWING_WIDTH];

    //Constructor
	public RLCrfControls(RLCrfCanvas canvas) {
		this.canvas = canvas;
		setLayout(new GridLayout(26, 4, 0, 2));
		setBackground(Color.lightGray);

		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));

		add(new Label("K : ", Label.RIGHT));
		add(tf_k = new TextField("1", 6));
		add(new Label(""));
		add(new Label(""));

		add(new Label(""));
		add(new Label("(n)", Label.CENTER));
		add(new Label(""));
		add(new Label("(n)", Label.CENTER));

		add(new Label("ZO : ", Label.RIGHT));
		add(tf_zo = new TextField("0", 6));
		add(new Label("PO : ", Label.RIGHT));
		add(tf_po = new TextField("0", 6));
		
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));

		add(new Label(""));
		add(new Label("(f [Hz])", Label.CENTER));
		add(new Label("(f [Hz])", Label.CENTER));
		add(new Label(""));

		add(new Label("ZR : ", Label.RIGHT));
		add(tf_zr[0] = new TextField("0", 6));
		add(tf_zr[1] = new TextField("0", 6));
		c_szr12.addItem("Pos.");
		c_szr12.addItem("Neg.");
		add(c_szr12);		

		add(new Label(""));		
		add(tf_zr[2] = new TextField("0", 6));
		add(tf_zr[3] = new TextField("0", 6));
		c_szr34.addItem("Pos.");
		c_szr34.addItem("Neg.");
		add(c_szr34);

		add(new Label("PR : ", Label.RIGHT));
		add(tf_pr[0] = new TextField("0", 6));
		add(tf_pr[1] = new TextField("0", 6));
		add(new Label(""));
		
		add(new Label(""));
		add(tf_pr[2] = new TextField("0", 6));
		add(tf_pr[3] = new TextField("0", 6));
		add(new Label(""));

		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));

		add(new Label(""));
		add(new Label("(f [Hz])", Label.CENTER));
		add(new Label("(Q) ", Label.CENTER));
		add(new Label(""));

		add(new Label("ZCC : ", Label.RIGHT));
		add(tf_zcc[0][0] = new TextField("0", 6));
		add(tf_zcc[0][1] = new TextField("0", 6));
		c_szcc1.addItem("Pos.");
		c_szcc1.addItem("Neg.");
		add(c_szcc1);		

		add(new Label(""));
		add(tf_zcc[1][0] = new TextField("0", 6));
		add(tf_zcc[1][1] = new TextField("0", 6));
		c_szcc2.addItem("Pos.");
		c_szcc2.addItem("Neg.");
		add(c_szcc2);		
		
		add(new Label("PCC : ", Label.RIGHT));
		add(tf_pcc[0][0] = new TextField("0", 6));
		add(tf_pcc[0][1] = new TextField("0", 6));
		add(new Label(""));

		add(new Label(""));
		add(tf_pcc[1][0] = new TextField("0", 6));
		add(tf_pcc[1][1] = new TextField("0", 6));
		add(new Label(""));

		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));

		add(new Label(""));
		add(new Label("(f [Hz])", Label.CENTER));
		add(new Label(""));
		add(new Label("(f [Hz])", Label.CENTER));

		add(new Label("Fmin : ", Label.RIGHT));
		add(tf_fmin = new TextField("1", 6));
		add(new Label("Fmax : ", Label.RIGHT));
		add(tf_fmax = new TextField("10", 6));
		
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));

		add(new Label(""));
		add(new Label(""));
		add(new Button("Executar"));
		add(new Label(""));

		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));		

		add(new Label("ERRO : ", Label.RIGHT));
		add(l_error1 = new Label("", Label.CENTER));
		add(l_error2 = new Label("", Label.LEFT));
		add(new Label(""));		

		add(new Label("hor : ", Label.RIGHT));
		add(new Label("[div/dc]", Label.CENTER));
		add(l_divx = new Label("0", Label.LEFT));
		add(new Label(""));
		
		add(new Label("ver : ", Label.RIGHT));
		add(new Label("[dB/div]", Label.CENTER));
		add(l_divy1 = new Label("0", Label.LEFT));
		add(new Label(""));		
		
		add(new Label("ver : ", Label.RIGHT));
		add(new Label("[rad/div]", Label.CENTER));
		add(l_divy2 = new Label("0", Label.LEFT));
		add(new Label(""));		
	}

    //Methods
	public boolean action(Event ev, Object arg) {
		if (ev.target instanceof Button) {
			evaluateParameters();
			return true;
		}
		return false;
    }

	void evaluateParameters() {
		
		int zo, po;
		float k, fmin, fmax;
		float[] zr = new float[4];//[f]
		float[] pr = new float[4];
		float[][] zcc = new float[2][2];//[f][Q]
		float[][] pcc = new float[2][2];
		boolean pos_szr12, pos_szr34, pos_szcc1, pos_szcc2;

		k = Float.valueOf(tf_k.getText().trim()).floatValue();
		
		zo = Integer.parseInt(tf_zo.getText().trim());
		po = Integer.parseInt(tf_po.getText().trim());
		
		zr[0] = Float.valueOf(tf_zr[0].getText().trim()).floatValue();
		zr[1] = Float.valueOf(tf_zr[1].getText().trim()).floatValue();
		zr[2] = Float.valueOf(tf_zr[2].getText().trim()).floatValue();
		zr[3] = Float.valueOf(tf_zr[3].getText().trim()).floatValue();
		
		pos_szr12 = c_szr12.getSelectedItem().equals("Pos.");
		pos_szr34 = c_szr34.getSelectedItem().equals("Pos.");
		
		pr[0] = Float.valueOf(tf_pr[0].getText().trim()).floatValue();
		pr[1] = Float.valueOf(tf_pr[1].getText().trim()).floatValue();
		pr[2] = Float.valueOf(tf_pr[2].getText().trim()).floatValue();
		pr[3] = Float.valueOf(tf_pr[3].getText().trim()).floatValue();
		
		zcc[0][0] = Float.valueOf(tf_zcc[0][0].getText().trim()).floatValue();
		zcc[0][1] = Float.valueOf(tf_zcc[0][1].getText().trim()).floatValue();
		zcc[1][0] = Float.valueOf(tf_zcc[1][0].getText().trim()).floatValue();
		zcc[1][1] = Float.valueOf(tf_zcc[1][1].getText().trim()).floatValue();
		
		pos_szcc1 = c_szcc1.getSelectedItem().equals("Pos.");
		pos_szcc2 = c_szcc2.getSelectedItem().equals("Pos.");
		
		pcc[0][0] = Float.valueOf(tf_pcc[0][0].getText().trim()).floatValue();
		pcc[0][1] = Float.valueOf(tf_pcc[0][1].getText().trim()).floatValue();
		pcc[1][0] = Float.valueOf(tf_pcc[1][0].getText().trim()).floatValue();
		pcc[1][1] = Float.valueOf(tf_pcc[1][1].getText().trim()).floatValue();
		
		fmin = Float.valueOf(tf_fmin.getText().trim()).floatValue();
		fmax = Float.valueOf(tf_fmax.getText().trim()).floatValue();

		//Reset messages
		l_error1.setText("");
		l_error2.setText("");
		l_divx.setText("0");
		l_divy1.setText("0");
		l_divy2.setText("0");
		
		//Tests values
		//If values are invalid, an error message is printed.
		if (k == 0)	{
			l_error1.setText("K = 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
	
		if (zo < 0)	{
			l_error1.setText("ZO");
			l_error2.setText("n < 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		if (po < 0)	{
			l_error1.setText("PO");
			l_error2.setText("n < 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		for (int i = 0; i < 4; i++) {
			if (zr[i] < 0) {
				l_error1.setText("ZR");
				l_error2.setText("f < 0");
				canvas.setDrawFlag(false);
				canvas.repaint();
				return;
			}
			if (pr[i] < 0) {
				l_error1.setText("PR");
				l_error2.setText("f < 0");
				canvas.setDrawFlag(false);
				canvas.repaint();
				return;
			}
		}
		
		for (int i = 0; i < 2; i++) {		
			if (zcc[i][0] < 0) {
				l_error1.setText("ZCC");
				l_error2.setText("f < 0");
				canvas.setDrawFlag(false);
				canvas.repaint();
				return;
			}
			if (pcc[i][0] < 0) {
				l_error1.setText("PCC");
				l_error2.setText("f < 0");
				canvas.setDrawFlag(false);
				canvas.repaint();
				return;
			}
		}
		
		if ((zcc[0][0] > 0 && zcc[0][1] <= 0) || (zcc[1][0] > 0 && zcc[1][1] <= 0)) {
			l_error1.setText("ZCC");
			l_error2.setText("Q < 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		if ((pcc[0][0] > 0 && pcc[0][1] <= 0) || (pcc[1][0] > 0 && pcc[1][1] <= 0)) {
			l_error1.setText("PCC");
			l_error2.setText("Q < 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		if (fmin <= 1e-4) {
			l_error1.setText("Fmin");
			l_error2.setText("f <= 1e-4");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		if (fmax <= fmin) {
			l_error1.setText("Fmax");
			l_error2.setText("f <= Fmin");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		if (checkInstability(zo, po, zr, pr, zcc, pcc)) {
			l_error1.setText("Sistema");
			l_error2.setText("Instvel");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		setFrequencies(fmin, fmax);
		execK(k);
		execZO(zo);
		execPO(po);
		execZR(zr, pos_szr12, pos_szr34);
		execPR(pr);
		execZCC(zcc, pos_szcc1, pos_szcc2);
		execPCC(pcc);
		canvas.realToScreen(this.l_divy1, this.l_divy2); 
	}
   
	boolean checkInstability(int zo, int po, float[] zr, float[] pr, float[][] zcc, float[][] pcc) {
		int numz = zo, nump = po;
	
		for (int i = 0; i < 4; i++) {
			if (zr[i] > 0)
				numz ++;
			if (pr[i] > 0)
				nump ++;
		}
		for (int i = 0; i < 2; i++) {
			if (zcc[i][0] > 0)
				numz ++;
			if (pcc[i][0] > 0)
				nump ++;
		}
		
		if (numz <= nump)
			return false;
		else 
			return true;
	}

	void setFrequencies(float fmin, float fmax) {
		float aux, dec;

		//fmin
		aux = fmin;
		dec = 0;
		if (aux < 1) {
			while (aux < 1) {
				aux = aux * 10;
				dec--;
			}
			fmin = (float) Math.pow(10, dec);
		}
		else if (aux >= 1 && aux < 10)
			fmin = 1;
		else {
			while (aux >= 10) {
				aux = aux / 10;
				dec++;
			}
			fmin = (float) Math.pow(10, dec);
		}
		tf_fmin.setText(Float.toString(fmin));
		
		//fmax
		aux = fmax;
		dec = 1;
		if (aux < 1) {
			while (aux < 1) {
				aux = aux * 10;
				dec--;
			}
			fmax = (float) Math.pow(10, dec);
		}
		else if (aux >= 1 && aux <= 10)
			fmax = 10;
		else {
			while (aux > 10) {
				aux = aux / 10;
				dec++;
			}
			fmax = (float) Math.pow(10, dec);
		}
		tf_fmax.setText(Float.toString(fmax));

		//Print value for X divisions
		dec = 1;
		aux = fmax / fmin;
		while (aux > 10) {
			aux = aux / 10;
			dec++;
		}		
		l_divx.setText(Float.toString(canvas.DRAWING_WIDTH / canvas.GRID_SIZE / dec));
		
		//Set log for each pixel
		float r = (float) Math.log(fmax / fmin);
		for (int i = 0; i < canvas.DRAWING_WIDTH; i++)
			freq[i] = fmin * (float) Math.exp(i * r / canvas.DRAWING_WIDTH);
	}
	
	void execK(float k) {
		for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
			//log 10 a = log e a / log e 10
			canvas.amp_pts[i] = 20 * (float) (Math.log(Math.abs(k)) / ln10);
			canvas.pha_pts[i] = 0;
		}
	}				
   
    void execZO(float zo) {
		if (zo > 0) { 
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) { 
				//log 10 a = log e a / log e 10
		   		canvas.amp_pts[i] += zo * 20 * (float) (Math.log(Math.abs(2 * Math.PI * freq[i])) / ln10);   
				canvas.pha_pts[i] += zo * (float) Math.PI;
			}
		}
    }

    void execPO(float po) {
		if (po > 0)	 {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= po * 20 * (float) (Math.log(Math.abs(2 * Math.PI * freq[i])) / ln10);   
				canvas.pha_pts[i] -= po * (float) Math.PI;
			}
		}
	}

	void execZR(float[] zr, boolean pos_szr12, boolean pos_szr34) {
		if (zr[0] > 0) {
			if (pos_szr12) {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[0], 2)) / ln10);   
					canvas.pha_pts[i] -= (float) Math.atan(freq[i] / zr[0]);
				}
			}
			else { 
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[0], 2)) / ln10);   
					canvas.pha_pts[i] += (float) Math.atan(freq[i] / zr[0]);
				}
			}
		}
		if (zr[1] > 0) {
			if (pos_szr12) {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[1], 2)) / ln10);   
					canvas.pha_pts[i] -= (float) Math.atan(freq[i] / zr[1]);
				}
			}
			else {
				 for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[1], 2)) / ln10);   
					canvas.pha_pts[i] += (float) Math.atan(freq[i] / zr[1]);
				}
			}
		}
		if (zr[2] > 0) {
			if (pos_szr34) {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[2], 2)) / ln10);   
					canvas.pha_pts[i] -= (float) Math.atan(freq[i] / zr[2]);
				}
			}
			else {
				 for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[2], 2)) / ln10);   
					canvas.pha_pts[i] += (float) Math.atan(freq[i] / zr[2]);
				}
			}
		}
		if (zr[3] > 0) {
			if (pos_szr34) {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[3], 2)) / ln10);   
					canvas.pha_pts[i] -= (float) Math.atan(freq[i] / zr[3]);
				}
			}
			else {
				 for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(1 + Math.pow(freq[i] / zr[3], 2)) / ln10);   
					canvas.pha_pts[i] += (float) Math.atan(freq[i] / zr[3]);
				}
			}
		}
	}

	void execPR(float[] pr) {
		if (pr[0] > 0) {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= 10 * (float) (Math.log(1 + Math.pow(freq[i] / pr[0], 2)) / ln10);   
				canvas.pha_pts[i] -= (float) (Math.atan(freq[i] / pr[0]));
			}
		}
		if (pr[1] > 0) {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= 10 * (float) (Math.log(1 + Math.pow(freq[i] / pr[1], 2)) / ln10);   
				canvas.pha_pts[i] -= (float) (Math.atan(freq[i] / pr[1]));
			}
		}
		if (pr[2] > 0) {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= 10 * (float) (Math.log(1 + Math.pow(freq[i] / pr[2], 2)) / ln10);   
				canvas.pha_pts[i] -= (float) (Math.atan(freq[i] / pr[2]));
			}
		}
		if (pr[3] > 0) {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= 10 * (float) (Math.log(1 + Math.pow(freq[i] / pr[3], 2)) / ln10);   
				canvas.pha_pts[i] -= (float) (Math.atan(freq[i] / pr[3]));
			}
		}
	}

	void execZCC(float[][] zcc, boolean pos_szcc1, boolean pos_szcc2) {
		if (zcc[0][0] > 0) {
			if (pos_szcc1) {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(Math.pow((1 - Math.pow(freq[i] / zcc[0][0], 2)) ,2) + Math.pow(freq[i] / zcc[0][0] / zcc[0][1], 2)) / ln10);   
					canvas.pha_pts[i] -= (float) Math.atan(freq[i] / zcc[0][0] / (zcc[0][1] * (1 - Math.pow(freq[i] / zcc[0][0], 2))));
				}
			}
			else {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(Math.pow((1 - Math.pow(freq[i] / zcc[0][0], 2)) ,2) + Math.pow(freq[i] / zcc[0][0] / zcc[0][1], 2)) / ln10);   
					canvas.pha_pts[i] += (float) Math.atan(freq[i] / zcc[0][0] / (zcc[0][1] * (1 - Math.pow(freq[i] / zcc[0][0], 2))));
				}
			}
		}
		if (zcc[1][0] > 0) {
			if (pos_szcc2) {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(Math.pow((1 - Math.pow(freq[i] / zcc[1][0], 2)) ,2) + Math.pow(freq[i] / zcc[1][0] / zcc[1][1], 2)) / ln10);   
					canvas.pha_pts[i] -= (float) Math.atan(freq[i] / zcc[1][0] / (zcc[1][1] * (1 - Math.pow(freq[i] / zcc[1][0], 2))));
				}
			}
			else {
				for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
					//log 10 a = log e a / log e 10
					canvas.amp_pts[i] += 10 * (float) (Math.log(Math.pow((1 - Math.pow(freq[i] / zcc[1][0], 2)) ,2) + Math.pow(freq[i] / zcc[1][0] / zcc[1][1], 2)) / ln10);   
					canvas.pha_pts[i] += (float) Math.atan(freq[i] / zcc[1][0] / (zcc[1][1] * (1 - Math.pow(freq[i] / zcc[1][0], 2))));
				}
			}
		}
	}
	
	void execPCC(float[][] pcc) {
		if (pcc[0][0] > 0) {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= 10 * (float) (Math.log(Math.pow((1 - Math.pow(freq[i] / pcc[0][0], 2)) ,2) + Math.pow(freq[i] / pcc[0][0] / pcc[0][1], 2)) / ln10);   
				canvas.pha_pts[i] -= (float) (Math.atan(freq[i] / pcc[0][0] / (pcc[0][1] * (1 - Math.pow(freq[i] / pcc[0][0], 2)))));
			}
		}
		if (pcc[1][0] > 0) {
			for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
				//log 10 a = log e a / log e 10
				canvas.amp_pts[i] -= 10 * (float) (Math.log(Math.pow((1 - Math.pow(freq[i] / pcc[1][0], 2)) ,2) + Math.pow(freq[i] / pcc[1][0] / pcc[1][1], 2)) / ln10);   
				canvas.pha_pts[i] -= (float) (Math.atan(freq[i] / pcc[1][0] / (pcc[1][1] * (1 - Math.pow(freq[i] / pcc[1][0], 2)))));
			}
		}
	}
}