import java.awt.*;
import java.applet.*;


//Class RLCsfc
public class RLCsfc extends Applet {
    
	//Fields
	RLCsfcControls controls;
	
	//Constructor
	public void init() {
		setLayout(new BorderLayout());
		RLCsfcCanvas c = new RLCsfcCanvas();
		add("Center", c);
		add("East", controls = new RLCsfcControls(c));
    }

    //Methods
	public void start() {
		controls.enable();
    }

    public void stop() {
		controls.disable();
    }

    public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY) {
			System.exit(0);
		}
		return false;
    }

    public static void main(String args[]) {
		Frame f = new Frame("Simulador da Soluo Forada Constante de Circuitos RLC-srie");
		RLCsfc	RLCsfcSimulator = new RLCsfc();

		RLCsfcSimulator.init();
		RLCsfcSimulator.start();

		f.add("Center", RLCsfcSimulator);
		f.resize(525, 390);
		f.show();
    }
}
    

//Class RLCsfcCanvas
class RLCsfcCanvas extends Canvas {
    
	//Fields
	final static int GRAPHICS_WIDTH = 360;
	final static int GRAPHICS_HEIGHT = 360;
	final static int DRAWING_WIDTH = 320;
	final static int DRAWING_HEIGHT = 320;
	final static int GRID_SIZE = 20;
	float[] output = new float[DRAWING_WIDTH];
	boolean draw_flag = false;//true when graph is valid
	
	//Constructor
	public RLCsfcCanvas() {
		setBackground(Color.white);
	}

	//Methods
	public void paint(Graphics g) {
		Rectangle r = new Rectangle(GRAPHICS_WIDTH, GRAPHICS_HEIGHT);

		//Grid
		int hlines = GRAPHICS_HEIGHT / GRID_SIZE;
		int vlines = GRAPHICS_WIDTH / GRID_SIZE;
		g.setColor(new Color(211, 211, 211));
		for (int i = 1; i < hlines; i++)
		    g.drawLine(0, i * GRID_SIZE, GRAPHICS_WIDTH, i * GRID_SIZE);
		for (int i = 1; i < vlines; i++)
			g.drawLine(i * GRID_SIZE, 0, i * GRID_SIZE, GRAPHICS_HEIGHT);
		
		//Border
		g.setColor(new Color(0, 0, 128));
		g.drawLine(0, 0, GRAPHICS_WIDTH, 0);
		g.drawLine(0, GRAPHICS_HEIGHT, GRAPHICS_WIDTH, GRAPHICS_HEIGHT);
		g.drawLine(0, 0, 0, GRAPHICS_HEIGHT);
		g.drawLine(GRAPHICS_WIDTH, 0, GRAPHICS_WIDTH, GRAPHICS_HEIGHT);

		//Axis
		g.setColor(Color.black);
		int[] xpoints = new int[3];
		int[] ypoints = new int[3];
		//Up Arrow
		xpoints[0] = 20;
		ypoints[0] = 0;
		xpoints[1] = 16;
		ypoints[1] = 8;
		xpoints[2] = 24;
		ypoints[2] = 8;
		g.fillPolygon(xpoints, ypoints, 3);
		//Right Arrow
		xpoints[0] = GRAPHICS_WIDTH;
		ypoints[0] = GRAPHICS_HEIGHT / 2;
		xpoints[1] = GRAPHICS_WIDTH - 8;
		ypoints[1] = GRAPHICS_HEIGHT / 2 - 4;
		xpoints[2] = GRAPHICS_WIDTH - 8;
		ypoints[2] = GRAPHICS_HEIGHT / 2 + 4;
		g.fillPolygon(xpoints, ypoints, 3);
		//X-axis
		g.drawLine(0, GRAPHICS_HEIGHT / 2, GRAPHICS_WIDTH, GRAPHICS_HEIGHT / 2);
		//Y-axis
		g.drawLine(20, 0, 20, GRAPHICS_HEIGHT);
		
		//Text
		Font graphics_font = new Font("Arial", Font.PLAIN, 12);
		g.setFont(graphics_font);
		g.drawString("t", GRAPHICS_WIDTH - 10, GRAPHICS_HEIGHT / 2 + 15); 
		g.setColor(Color.red);
		g.drawString("v (t)", 25, 15); 
    
		//Drawing
		if (draw_flag) {
			for (int i = 0; i < 20; i++)
				//Initial condition
				g.drawLine(i, (int) output[0], i, (int) output[0]);
			for (int i = 0; i < DRAWING_WIDTH; i++)
				g.drawLine(i + 20, (int) output[i], i + 20, (int) output[i]);
		}
	}
	
	void setDrawFlag(boolean draw_flag) {
		this.draw_flag = draw_flag;
	}

	void realToScreen(float inc, Label x, Label y) {
		float max_output = 0, gain = 0, offset;

		for (int i = 0; i < DRAWING_WIDTH; i++) {
			//Computing	maximum output for graph auto-arrange in Y
			if (Math.abs(output[i]) > max_output)
				max_output = Math.abs(output[i]);
		}

		//Values for conversion	from real output to screen coordinates
		//max_output = 0 if v(0) = i(0) = 0
		if (max_output > 0) {
			gain = DRAWING_WIDTH / (2 * max_output);
			//Print values for Y divisions
			y.setText(Float.toString((1 / gain) * GRID_SIZE));
		}
		offset = DRAWING_WIDTH / 2 + 20;

		//Print values for X divisions
		x.setText(Float.toString(inc * GRID_SIZE));
		
		//Conversion from real output values to screen coordinates
		for (int i = 0; i < DRAWING_WIDTH; i++)
			output[i] = offset - gain * output[i];			
		
		//Draw graphics
		setDrawFlag(true);
		repaint();
	}
}


//Class RLCsfcControls
class RLCsfcControls extends Panel {
    
	//Fields
	RLCsfcCanvas canvas;
	TextField tf_r, tf_l, tf_c, tf_vs, tf_v0, tf_i0, tf_t;
	Label l_solution, l_q, l_error, l_divx, l_divy;

    //Constructor
	public RLCsfcControls(RLCsfcCanvas canvas) {
		this.canvas = canvas;
		setLayout(new GridLayout(16, 2, 0, 2));
		setBackground(Color.lightGray);

		add(new Label(""));
		add(new Label(""));
		add(new Label("R [\u03A9] : ", Label.RIGHT));
		add(tf_r = new TextField("0.5", 8));
		add(new Label("L [H] : ", Label.RIGHT));
		add(tf_l = new TextField("1", 8));
		add(new Label("C [F] : ", Label.RIGHT));
		add(tf_c = new TextField("1", 8));
		add(new Label("Vs [V] : ", Label.RIGHT));
		add(tf_vs = new TextField("1", 8));
		add(new Label("v (0) [V] : ", Label.RIGHT));
		add(tf_v0 = new TextField("0", 8));
		add(new Label("i (0) [A] : ", Label.RIGHT));
		add(tf_i0 = new TextField("0", 8));
		add(new Label("t [s] : ", Label.RIGHT));
		add(tf_t = new TextField("20", 8));
		add(new Label(""));
		add(new Label(""));
		add(new Label(""));
		add(new Button("Executar"));
		add(new Label(""));
		add(new Label(""));
		add(new	Label("SOLUO : ", Label.RIGHT));
		add(l_solution = new Label("", Label.LEFT));
		add(new	Label("Q : ", Label.RIGHT));
		add(l_q = new Label("", Label.LEFT));		
		add(new	Label("ERRO : ", Label.RIGHT));
		add(l_error = new Label("", Label.LEFT));
		add(new	Label("hor [s/div] : ", Label.RIGHT));
		add(l_divx = new	Label("0", Label.LEFT));	   
		add(new Label("ver [V/div] : ", Label.RIGHT));
		add(l_divy = new	Label("0", Label.LEFT));	   
	}

    //Methods
	public boolean action(Event ev, Object arg) {
		if (ev.target instanceof Button) {
			evaluateParameters();
			return true;
		}
		return false;
    }

	void evaluateParameters() {
		float r, l, c, vs, v0, i0, t;
		float alfa, omega0;

		r = Float.valueOf(tf_r.getText().trim()).floatValue();
		l = Float.valueOf(tf_l.getText().trim()).floatValue();
		c = Float.valueOf(tf_c.getText().trim()).floatValue();
		vs = Float.valueOf(tf_vs.getText().trim()).floatValue();
		v0 = Float.valueOf(tf_v0.getText().trim()).floatValue();
		i0 = Float.valueOf(tf_i0.getText().trim()).floatValue();
		t = Float.valueOf(tf_t.getText().trim()).floatValue();

		//Reset messages
		l_solution.setText("");
		l_q.setText("");
		l_error.setText("");
		l_divx.setText("0");
		l_divy.setText("0");
		
		//Tests values of t, r, l, c.
		//If values are invalid, an l_error message is printed.
		if (t <= 0) {
			l_error.setText("t <= 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}
		
		if (r <= 0) {
			l_error.setText("R <= 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}

		if (l <= 0 || c <= 0) {
			l_error.setText("L ou C <= 0");
			canvas.setDrawFlag(false);
			canvas.repaint();
			return;
		}

		// alfa=R/2L, omega0=1/sqrt(LC)
		alfa = r / (2 * l);
		omega0 = 1 / (float) Math.sqrt(l * c);

		//l_solution type evaluation
		//Prints l_solution type and Q factor
		if (alfa > omega0) {
			l_solution.setText("Sobre-Amort.");
			l_q.setText(Float.toString(omega0 / (2 * alfa)));
			execSobreAmort(alfa, omega0, v0, i0, c, vs, t);
		}
		else if (alfa == omega0) {
			l_solution.setText("Critic. Amort.");
			l_q.setText(Float.toString(omega0 / (2 * alfa)));
			execCriticAmort(alfa, v0, i0, c, vs, t);
		}
		//alfa < omega0
		else {
			l_solution.setText("Sub-Amort.");
			l_q.setText(Float.toString(omega0 / (2 * alfa)));
			execSubAmort(alfa, omega0, v0, i0, c, vs, t);
		}
		return;
	}

	void execSobreAmort(float alfa, float omega0, float v0, float i0, float c, float vs, float t) {
		float omegad, s1, s2, A1, A2;
		float inc, ti;

		omegad = (float) Math.sqrt(alfa * alfa - omega0 * omega0);
		
		s1 = - alfa + omegad;
		s2 = - alfa - omegad;

		A1 = (s2 * c * (v0 - vs) - i0) / (c * (s2 - s1));
		A2 = (s1 * c * (vs - v0) + i0) / (c * (s2 - s1));
			
		//Time increment
		inc = t / canvas.DRAWING_WIDTH;
		//Time at instant i
		ti = 0;
		
		for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
			canvas.output[i] = (float) (A1 * Math.exp(s1 * ti) + A2 * Math.exp(s2 * ti) + vs);
			ti += inc;
		}
		canvas.realToScreen(inc, this.l_divx, this.l_divy);
	}

	void execCriticAmort(float alfa, float v0, float i0, float c, float vs, float t) {
		float A1, A2;
		float inc, ti;

		A1 = v0 - vs;
		A2 = i0 / c + alfa * c * (v0 - vs);

		//Time increment
		inc = t / canvas.DRAWING_WIDTH;
		//Time at instant i
		ti = 0;
	
		for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
			canvas.output[i] = (float) ((A1 + A2 * ti) *  Math.exp(- alfa * ti) + vs);
			ti += inc;
		}
		canvas.realToScreen(inc, this.l_divx, this.l_divy);
	}

	void execSubAmort(float alfa, float omega0, float v0, float i0, float c, float vs, float t) {
		float omegad, A3 = 0, theta = 0;
		float inc, ti;
		
		omegad = (float) Math.sqrt(omega0 * omega0 - alfa * alfa);

		if (v0 != vs) {
			theta = (float) Math.atan((i0 + alfa * c * (v0 - vs)) / (omegad * c * (v0 - vs)));
			A3 = (v0 - vs) / (float) Math.cos(theta);
		}

		//Time increment
		inc = t / canvas.DRAWING_WIDTH;
		//Time at instant i
		ti = 0;

		for (int i = 0; i < canvas.DRAWING_WIDTH; i++) {
			//If v0 = vs then output = vs.
			canvas.output[i] = (float) (Math.exp(- alfa * ti) * A3 * Math.cos(omegad * ti - theta) + vs);
			ti += inc;
		}
		canvas.realToScreen(inc, this.l_divx, this.l_divy);
	}
}
